@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,17 +32,17 @@
 #include <sys/stat.h>
 #include <sys/file.h>
 
-#include <lxc/lxc.h>
-#include <lxc/log.h>
-#include <lxc/start.h>
-#include <lxc/cgroup.h>
-#include <lxc/monitor.h>
+#include "lxc.h"
+#include "log.h"
+#include "start.h"
+#include "cgroup.h"
+#include "monitor.h"
 #include "commands.h"
 #include "config.h"
 
 lxc_log_define(lxc_state, lxc);
 
-static char *strstate[] = {
+static const char * const strstate[] = {
 	"STOPPED", "STARTING", "RUNNING", "STOPPING",
 	"ABORTING", "FREEZING", "FROZEN", "THAWED",
 };
@@ -56,7 +56,8 @@ const char *lxc_state2str(lxc_state_t state)
 
 lxc_state_t lxc_str2state(const char *state)
 {
-	int i, len;
+	size_t len;
+	lxc_state_t i;
 	len = sizeof(strstate)/sizeof(strstate[0]);
 	for (i = 0; i < len; i++)
 		if (!strcmp(strstate[i], state))
@@ -66,106 +67,16 @@ lxc_state_t lxc_str2state(const char *state)
 	return -1;
 }
 
-static int freezer_state(const char *name, const char *lxcpath)
-{
-	char *nsgroup;
-	char freezer[MAXPATHLEN];
-	char status[MAXPATHLEN];
-	FILE *file;
-	int err;
-
-	err = lxc_cgroup_path_get(&nsgroup, "freezer", name, lxcpath);
-	if (err)
-		return -1;
-
-	err = snprintf(freezer, MAXPATHLEN, "%s/freezer.state", nsgroup);
-	if (err < 0 || err >= MAXPATHLEN)
-		return -1;
-
-	file = fopen(freezer, "r");
-	if (!file)
-		return -1;
-
-	err = fscanf(file, "%s", status);
-	fclose(file);
-
-	if (err == EOF) {
-		SYSERROR("failed to read %s", freezer);
-		return -1;
-	}
-
-	return lxc_str2state(status);
-}
-
-static lxc_state_t __lxc_getstate(const char *name, const char *lxcpath)
-{
-	struct lxc_command command = {
-		.request = { .type = LXC_COMMAND_STATE },
-	};
-
-	int ret, stopped = 0;
-
-	ret = lxc_command(name, &command, &stopped, lxcpath);
-	if (ret < 0 && stopped)
-		return STOPPED;
-
-	if (ret < 0) {
-		ERROR("failed to send command");
-		return -1;
-	}
-
-	if (!ret) {
-		WARN("'%s' has stopped before sending its state", name);
-		return -1;
-	}
-
-	if (command.answer.ret < 0) {
-		ERROR("failed to get state for '%s': %s",
-			name, strerror(-command.answer.ret));
-		return -1;
-	}
-
-	DEBUG("'%s' is in '%s' state", name, lxc_state2str(command.answer.ret));
-
-	return command.answer.ret;
-}
-
 lxc_state_t lxc_getstate(const char *name, const char *lxcpath)
 {
-	int state = freezer_state(name, lxcpath);
+	extern lxc_state_t freezer_state(const char *name, const char *lxcpath);
+
+	lxc_state_t state = freezer_state(name, lxcpath);
 	if (state != FROZEN && state != FREEZING)
-		state = __lxc_getstate(name, lxcpath);
+		state = lxc_cmd_get_state(name, lxcpath);
 	return state;
 }
 
-/*----------------------------------------------------------------------------
- * functions used by lxc-start mainloop
- * to handle above command request.
- *--------------------------------------------------------------------------*/
-extern int lxc_state_callback(int fd, struct lxc_request *request,
-			struct lxc_handler *handler)
-{
-	struct lxc_answer answer;
-	int ret;
-
-	memset(&answer, 0, sizeof(answer));
-	answer.ret = handler->state;
-
-	ret = send(fd, &answer, sizeof(answer), 0);
-	if (ret < 0) {
-		WARN("failed to send answer to the peer");
-		goto out;
-	}
-
-	if (ret != sizeof(answer)) {
-		ERROR("partial answer sent");
-		goto out;
-	}
-
-out:
-	return ret;
-}
-
 static int fillwaitedstates(const char *strstates, int *states)
 {
 	char *token, *saveptr = NULL;
@@ -201,6 +112,9 @@ extern int lxc_wait(const char *lxcname, const char *states, int timeout, const
 	if (fillwaitedstates(states, s))
 		return -1;
 
+	if (lxc_monitord_spawn(lxcpath))
+		return -1;
+
 	fd = lxc_monitor_open(lxcpath);
 	if (fd < 0)
 		return -1;
@@ -230,8 +144,11 @@ extern int lxc_wait(const char *lxcname, const char *states, int timeout, const
 				goto out_close;
 			curtime = tv.tv_sec;
 		}
-		if (lxc_monitor_read_timeout(fd, &msg, timeout) < 0)
-			goto out_close;
+		if (lxc_monitor_read_timeout(fd, &msg, timeout) < 0) {
+			/* try again if select interrupted by signal */
+			if (errno != EINTR)
+				goto out_close;
+		}
 
 		if (timeout != -1) {
 			retval = gettimeofday(&tv, NULL);
