@@ -1,4 +1,5 @@
 #
+# -*- coding: utf-8 -*-
 # python-lxc: Python bindings for LXC
 #
 # (C) Copyright Canonical Ltd. 2012
@@ -32,7 +33,7 @@ get_global_config_item = _lxc.get_global_config_item
 version = _lxc.get_version()
 
 
-class ContainerNetwork():
+class ContainerNetwork(object):
     props = {}
 
     def __init__(self, container, index):
@@ -96,8 +97,12 @@ class ContainerNetwork():
         return self.__set_network_item(self.props[key], value)
 
     def __clear_network_item(self, key):
-        return self.container.clear_config_item("lxc.network.%s.%s" % (
-                                                self.index, key))
+        if key in ("ipv4", "ipv6"):
+            return self.container.clear_config_item("lxc.network.%s.%s" % (
+                                                    self.index, key))
+        else:
+            return self.container.set_config_item("lxc.network.%s.%s" % (
+                                                    self.index, key), "")
 
     def __get_network_item(self, key):
         return self.container.get_config_item("lxc.network.%s.%s" % (
@@ -200,11 +205,11 @@ class Container(_lxc.Container):
 
         return _lxc.Container.set_config_item(self, key, value)
 
-    def create(self, template, flags=0, args=()):
+    def create(self, template=None, flags=0, args=(), bdevtype=None):
         """
             Create a new rootfs for the container.
 
-            "template" must be a valid template name.
+            "template" if passed must be a valid template name.
 
             "flags" (optional) is an integer representing the optional
             create flags to be passed.
@@ -212,17 +217,19 @@ class Container(_lxc.Container):
             "args" (optional) is a tuple of arguments to pass to the
             template. It can also be provided as a dict.
         """
-
         if isinstance(args, dict):
-            template_args = []
+            tmp_args = []
             for item in args.items():
-                template_args.append("--%s" % item[0])
-                template_args.append("%s" % item[1])
-        else:
-            template_args = args
-
-        return _lxc.Container.create(self, template=template,
-                                     flags=flags, args=tuple(template_args))
+                tmp_args.append("--%s" % item[0])
+                tmp_args.append("%s" % item[1])
+        template_args = {}
+        if template:
+    	    template_args['template'] = template
+        template_args['flags'] = flags
+        template_args['args'] = tuple(tmp_args)
+        if bdevtype:
+            template_args['bdevtype'] = bdevtype
+        return _lxc.Container.create(self, **template_args)
 
     def clone(self, newname, config_path=None, flags=0, bdevtype=None,
               bdevdata=None, newsize=0, hookargs=()):
